<?php
/**
 * UserRegistrationContentRestriction Functions.
 *
 * General core functions available on both the front-end and admin.
 *
 * @author   WPEverest
 * @category Core
 * @package  UserRegistrationContentRestriction/Functions
 * @version  1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * @param $path
 *
 * @return string
 */


function urcr_is_compatible() {

	$plugins_path = WP_PLUGIN_DIR . URCR_DS . 'user-registration' . URCR_DS . 'user-registration.php';

	if ( ! file_exists( $plugins_path ) ) {

		return __( 'Please install <code>user-registration</code> plugin to use <code>user-registration-content-restriction</code> addon.', 'user-registration-content-restriction' );
	}
	$plugin_file_path = 'user-registration/user-registration.php';

	include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

	if ( ! is_plugin_active( $plugin_file_path ) ) {

		return __( 'Please activate <code>user-registration</code> plugin to use <code>user-registration-content-restriction</code> addon.', 'user-registration-content-restriction' );

	}
	if ( function_exists( 'UR' ) ) {

		$user_registration_version = UR()->version;

	} else {

		$user_registration_version = get_option( 'user_registration_version' );

	}

	if ( version_compare( $user_registration_version, '1.1.0', '<' ) ) {

		return __( 'Please update your <code>user-registration</code> plugin(to at least 1.1.0 version) to use <code>user-registration-content-restriction</code> addon.', 'user-registration-content-restriction' );

	}

	return 'YES';

}

function urcr_check_plugin_compatibility() {

	add_action( 'admin_notices', 'urcr_admin_notices', 10 );

}

function urcr_admin_notices() {

	$class = 'notice notice-error';

	$message = urcr_is_compatible();

	if ( 'YES' !== $message ) {

		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), ( $message ) );
	}
}

function urcr_settings() {

	return apply_filters(
		'user_registration_content_restriction_settings', array(

			array(
				'title' => __( 'Content Restriction Settings', 'user-registration-content-restriction' ),
				'type'  => 'title',
				'desc'  => '',
				'id'    => 'user_registration_content_restriction_settings',
			),

			array(
				'row_class' => 'urcr_enable_disable urcr_content_restriction_enable',
				'title'     => __( 'Enable Content Restriction ?', 'user-registration-content-restriction' ),
				'desc'      => __( 'Check To Enable Content Restriction', 'user-registration-content-restriction' ),
				'id'        => 'user_registration_content_restriction_enable',
				'default'   => 'yes',
				'type'      => 'checkbox',
				'autoload'  => false,
			),
		
			array(
				'row_class' => 'urcr_content_restriction_allow_access_to',
				'title'    => __( 'Allow Access To', 'user-registration-content-restriction' ),
				'desc'     => __( 'Select Option To Allow Access To', 'user-registration-content-restriction' ),
				'id'       => 'user_registration_content_restriction_allow_access_to',
				'type'     => 'select',
				'class'    => 'ur-enhanced-select',
				'css'      => 'min-width: 350px;',
				'desc_tip' => true,
				'options'  => array( 'All Logged In Users','Choose Specific Roles', 'Guest Users' ),
			),

			array(
				'row_class' => 'urcr_content_restriction_allow_access_to_roles',
				'title'    => __( 'Select Roles', 'user-registration-content-restriction' ),
				'desc'     => __( 'The roles selected here will have access to restricted content.', 'user-registration-content-restriction' ),
				'id'       => 'user_registration_content_restriction_allow_to_roles',
				'default'  => array( 'administrator' ),
				'type'     => 'multiselect',
				'class'    => 'ur-enhanced-select',
				'css'      => 'min-width: 350px;',
				'desc_tip' => true,
				'options'  => urcr_get_all_roles(),
			),

			array(
				'title'    => __( 'Restricted Content Message', 'user-registration-content-restriction' ),
				'desc'     => __( 'The message you would like to display in restricted content.', 'user-registration-content-restriction' ),
				'id'       => 'user_registration_content_restriction_message',
 				'type'     => 'tinymce',
 				'default'  => 'This content is restricted!',
				'css'      => 'min-width: 350px;',
				'desc_tip' => true,
			),

			array(
				'type' => 'sectionend',
				'id'   => 'user_registration_content_restriction_settings',
			),
		)
	);
}

function urcr_get_allow_options() {
	
	return apply_filters( 'user_registration_content_restriction_to_roles', $all_roles );
}


function urcr_get_all_roles() {
	global $wp_roles;

	if ( ! class_exists( 'WP_Roles' ) ) {
		return;
	}

	$roles = array();
	if ( ! isset( $wp_roles ) ) {
		$wp_roles = new WP_Roles();
	}
	$roles = $wp_roles->roles;

	$all_roles = array();

	foreach ( $roles as $role_key => $role ) {

		$all_roles[ $role_key ] = $role['name'];
	}

	return apply_filters( 'user_registration_content_restriction_to_roles', $all_roles );
}


/**
 * Clean variables using sanitize_text_field. Arrays are cleaned recursively.
 * Non-scalar values are ignored.
 *
 * @param  string|array $var Data to sanitize.
 * @return string|array
 */
function urcr_clean( $var ) {
	if ( is_array( $var ) ) {
		return array_map( 'urcr_clean', $var );
	} else {
		return is_scalar( $var ) ? sanitize_text_field( $var ) : $var;
	}
}

