<?php
/**
 * User_Registration_Geolocation_Process
 * @package  User_Registration_Geolocation_Process
 * @since  1.0.0 
 */

defined( 'ABSPATH' ) || exit;	

/**
 * @class User_Registration_Geolocation_Process
 */
Class User_Registration_Geolocation_Process {

	/**
	 * API endpoints for looking up user IP address.
	 *
	 * @var array
	 */
	private $ip_lookup_apis = array(
		'ipify'             => 'http://api.ipify.org/',
		'ipecho'            => 'http://ipecho.net/plain',
		'ident'             => 'http://ident.me',
		'whatismyipaddress' => 'http://bot.whatismyipaddress.com',
	);

	/**
	 * API endpoints for geolocating an IP address
	 *
	 * @var array
	 */
	private $geoip_apis = array(
		'ipapi.co'   => 'https://ipapi.co/%s/json',
		'ipinfo.io'  => 'https://ipinfo.io/%s/json',
		'ip-api.com' => 'http://ip-api.com/json/%s',
	);

	/**
	 * User_Registration_Geolocation_Process Constructor
	 */
	public function __construct() {
		add_action( 'user_registration_after_register_user_action', array( $this, 'save_user_location' ), 5, 3 );
		add_action( 'user_registration_after_user_extra_information', array( $this, 'display_location' ) );
		add_filter( 'user_registration_process_smart_tag', array( $this, 'process_smart_tag' ), 10, 4 );
	}

	/**
	 * Get current user IP Address.
	 *
	 * @return string
	 */
	private function get_ip_address() {
		if ( isset( $_SERVER['HTTP_X_REAL_IP'] ) ) { // WPCS: input var ok, CSRF ok.
			return sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) );  // WPCS: input var ok, CSRF ok.
		} elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) { // WPCS: input var ok, CSRF ok.
			// Proxy servers can send through this header like this: X-Forwarded-For: client1, proxy1, proxy2
			// Make sure we always only send through the first IP in the list which should always be the client IP.
			return (string) rest_is_ip_address( trim( current( preg_split( '/[,:]/', sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) ) ) ) ); // WPCS: input var ok, CSRF ok.
		} elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) { // @codingStandardsIgnoreLine
			return sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ); // @codingStandardsIgnoreLine
		}
		return '';
	}

	/**
	 * Get user IP Address using an external service.
	 * This is used mainly as a fallback for users on localhost where
	 * get_ip_address() will be a local IP and non-geolocatable.
	 *
	 * @return string
	 */
	private function get_external_ip_address() {
		$external_ip_address = '0.0.0.0';

		if ( '' !== $this->get_ip_address() ) {
			$transient_name      = 'external_ip_address_' . $this->get_ip_address();
			$external_ip_address = get_transient( $transient_name );
		}

		if ( false === $external_ip_address ) {
			$external_ip_address     = '0.0.0.0';
			$ip_lookup_services      = apply_filters( 'user_registration_geolocation_ip_lookup_apis', $this->ip_lookup_apis );
			$ip_lookup_services_keys = array_keys( $ip_lookup_services );
			shuffle( $ip_lookup_services_keys );

			foreach ( $ip_lookup_services_keys as $service_name ) {
				$service_endpoint = $ip_lookup_services[ $service_name ];
				$response         = wp_safe_remote_get( $service_endpoint, array( 'timeout' => 2 ) );

				if ( ! is_wp_error( $response ) && rest_is_ip_address( $response['body'] ) ) {
					$external_ip_address = apply_filters( 'user_registration_geolocation_ip_lookup_api_response', ur_clean( $response['body'] ), $service_name );
					break;
				}
			}

			set_transient( $transient_name, $external_ip_address, WEEK_IN_SECONDS );
		}

		return $external_ip_address;
	}

	/**
	 * Geolocate an IP address.
	 *
	 * @param  string $ip_address   IP Address.
	 * @param  bool   $fallback     If true, fallbacks to alternative IP detection (can be slower).
	 * @param  bool   $api_fallback If true, uses geolocation APIs if the database file doesn't exist (can be slower).
	 * @return array
	 */
	private function geolocate_ip( $ip_address = '', $fallback = true, $api_fallback = true ) {
		// Filter to allow custom geolocation of the IP address.
		$location_data = apply_filters( 'user_registration_geolocate_ip', array(), $ip_address, $fallback, $api_fallback );

		if ( empty( $location_data ) ) {
			$ip_address = $ip_address ? $ip_address : $this->get_ip_address();

			if ( $api_fallback ) {
				$location_data = $this->geolocate_via_api( $ip_address );
			} else {
				$location_data = array();
			}

			if ( empty( $location_data['country'] ) && $fallback ) {
				// May be a local environment - find external IP.
				return $this->geolocate_ip( $this->get_external_ip_address(), false, $api_fallback );
			}
		}

		return $location_data;
	}


	/**
	 * Use APIs to Geolocate the user.
	 *
	 * @param  string $ip_address IP address.
	 * @return string|bool
	 */
	private function geolocate_via_api( $ip_address ) {
		$location_data = get_transient( 'geoip_' . $ip_address );

		if ( false === $location_data ) {
			$geoip_data          = array();
			$geoip_services      = apply_filters( 'user_registration_geolocation_geoip_apis', $this->geoip_apis );
			$geoip_services_keys = array_keys( $geoip_services );
			shuffle( $geoip_services_keys );

			foreach ( $geoip_services_keys as $service_name ) {
				$service_endpoint = $geoip_services[ $service_name ];
				$response         = wp_safe_remote_get( sprintf( $service_endpoint, $ip_address ), array( 'timeout' => 2 ) );

				if ( ! is_wp_error( $response ) && $response['body'] ) {
					switch ( $service_name ) {
						case 'ipinfo.io-':
							$data                       = json_decode( $response['body'] );
							$lat_log                    = isset( $data->loc ) ? explode( ',', $data->loc ) : array();
							$geoip_data['country']      = isset( $data->country ) ? $data->country : '';
							$geoip_data['country_code'] = isset( $data->country ) ? $data->country : '';
							$geoip_data['city']         = isset( $data->city ) ? $data->city : '';
							$geoip_data['region']       = isset( $data->region ) ? $data->region : '';
							$geoip_data['postal']       = isset( $data->postal ) ? $data->postal : '';
							$geoip_data['latitude']     = isset( $lat_log[0] ) ? $lat_log[0] : '';
							$geoip_data['longitude']    = isset( $lat_log[1] ) ? $lat_log[1] : '';
							break;
						case 'ip-api.com':
							$data                       = json_decode( $response['body'] );
							$geoip_data['country']      = isset( $data->country ) ? $data->country : '';
							$geoip_data['country_code'] = isset( $data->countryCode ) ? $data->countryCode : ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.NotSnakeCaseMemberVar
							$geoip_data['city']         = isset( $data->city ) ? $data->city : '';
							$geoip_data['region']       = isset( $data->regionName ) ? $data->regionName : ''; // phpcs:ignore WordPress.NamingConventions.ValidVariableName.NotSnakeCaseMemberVar
							$geoip_data['postal']       = isset( $data->postal ) ? $data->postal : '';
							$geoip_data['latitude']     = isset( $data->lat ) ? $data->lat : '';
							$geoip_data['longitude']    = isset( $data->lon ) ? $data->lon : '';
							break;
						case 'ipapi.co-':
							$data                       = json_decode( $response['body'] );
							$geoip_data['country']      = isset( $data->country_name ) ? $data->country_name : '';
							$geoip_data['country_code'] = isset( $data->country ) ? $data->country : '';
							$geoip_data['city']         = isset( $data->city ) ? $data->city : '';
							$geoip_data['region']       = isset( $data->region ) ? $data->region : '';
							$geoip_data['postal']       = isset( $data->postal ) ? $data->postal : '';
							$geoip_data['latitude']     = isset( $data->latitude ) ? $data->latitude : '';
							$geoip_data['longitude']    = isset( $data->longitude ) ? $data->longitude : '';
							break;
						default:
							$geoip_data = apply_filters( 'user_registration_geolocation_geoip_response_' . $service_name, array(), $response['body'] );
							break;
					}

					$location_data = array_map( 'sanitize_text_field', $geoip_data );

					if ( ! empty( $location_data['country'] ) ) {
						break;
					}
				}
			}

			set_transient( 'geoip_' . $ip_address, $location_data, WEEK_IN_SECONDS );
		}

		return $location_data;
	}

	/**
	 * Save users location in usermeta table with meta key ur_geolocation
	 * @return [type] [description]
	 */
	public function save_user_location( $form_data, $form_id, $user_id, $ip_address='', $fallback = true ) {

		/**
		 * Return if user or form id is not set.
		 */
		if ( ! isset( $form_id ) || ! isset( $user_id ) ) {
			return;
		}

		$location = $this->geolocate_ip( $ip_address, $fallback );

		if ( ! empty( $location['country'] ) ) {

			update_user_meta( $user_id, 'user_registration_geolocation', json_encode( $location ) );
			return true;
		}

		return false;
	}

	/**
	 * Display location on google map of user
	 * @param  obj $user 
	 * @return void
	 */
	public function display_location( $user = '' ) {

		if( empty( $user ) ) {
			return;
		}

		$location = get_user_meta( $user->ID, 'user_registration_geolocation', true );
		$location = $location ? json_decode( $location, true ) : array();

 		if ( ! empty( $location ) ) {
			$google_map_url = add_query_arg(
				array(
					'q'      => $location['city'] . ',' . $location['region'],
					'll'     => $location['latitude'] . ',' . $location['longitude'],
					'z'      => apply_filters( 'user_registration_geolocation_map_zoom', '6' ),
					'output' => 'embed',
				),
				'https://maps.google.com/maps'
			);

			$location_data = array(
					'country' 	=> esc_html__( 'Country', 'user-registration-geolocation' ),
					'city' 		=> esc_html__( 'City', 'user-registration-geolocation' ),
					'region'	=> esc_html__( 'Region', 'user-registration-geolocation' ),
					'postal'	=> esc_html__( 'Postal', 'user-registration-geolocation' ),
					'latitude'	=> esc_html__( 'Latitude', 'user-registration-geolocation' ),
					'longitude'	=> esc_html__( 'Longitude', 'user-registration-geolocation' ),
				);

			?>
			<h3><?php _e( 'Geolocation Data', 'user-registration-geolocation' );?></h3>
				<table class="form-table">
					<?php foreach( $location_data as $key => $label ) {

							if ( empty( $location[ $key ] ) ) {
								continue;
							}

							echo '<tr>
									<th><label for="'. $key .'">'. $label .'</label>
									</th>
									<td>
										'. esc_html( $location[ $key ] ) .'
									</td>
								</tr>';
						}
					?>
					<tr>
						<th>
						<iframe style="border: 5px solid white; max-width: 570px; width: 280%;" frameborder="" src="<?php echo esc_url( $google_map_url ); ?>"></iframe>
					</th>
					</tr>
				</table>
			<?php
		}
	}

	/**
	 * Send Geolocation data with google map to email with {{geolocation}} smarttag
	 * @param $form_data form filled data
	 * param  $form_id form id
	 * @param string $name_value smart tag and value to replace
	 * @param int $user_id user id
	 * @return array
	 */
	public function process_smart_tag( $name_value, $form_data, $form_id, $user_id ) {

		if( empty( $user_id ) ) {
			return $name_value;
		}

		$location = get_user_meta( $user_id, 'user_registration_geolocation', true );
		$location = $location ? json_decode( $location, true ) : array();

 		if ( ! empty( $location ) ) {
			$google_map_url = add_query_arg(
				array(
					'q'      => $location['city'] . ',' . $location['region'],
					'll'     => $location['latitude'] . ',' . $location['longitude'],
					'z'      => apply_filters( 'user_registration_geolocation_map_zoom', '6' ),
				),
				'https://maps.google.com/maps'
			);

			$img_src = add_query_arg(
				array(
					'center'  => $location['city'] . ',' . $location['region'],
					'll'      => $location['latitude'] . ',' . $location['longitude'],
					'size'    => '300x100',
					'maptype' => 'roadmap',
					'zoom'    => apply_filters( 'user_registration_geolocation_map_zoom', '6' ),
					'markers' => 'color:red%7C' . $location['latitude'] . ',' . $location['longitude'],
				),
				'https://maps.googleapis.com/maps/api/staticmap'
			);

			$location_data = array(
				'country' 	=> esc_html__( 'Country', 'user-registration-geolocation' ),
				'city' 		=> esc_html__( 'City', 'user-registration-geolocation' ),
				'region'	=> esc_html__( 'Region', 'user-registration-geolocation' ),
				'postal'	=> esc_html__( 'Postal', 'user-registration-geolocation' ),
				'latitude'	=> esc_html__( 'Latitude', 'user-registration-geolocation' ),
				'longitude'	=> esc_html__( 'Longitude', 'user-registration-geolocation' ),
			);

			$value = '<h3>'. __( 'Geolocation Data:', 'user-registration-geolocation' ) .'</h3>';

			foreach( $location_data as $key => $label ){

				if ( empty( $location[ $key ] ) ) {
					continue;
				}

				$value .= '<table><tr>
						<th><label for="'. $key .'">'. $label .'</label>
						</th>
						<td>
							'. esc_html( $location[ $key ] ) .'
						</td>
					</tr></table>';
			}

			$value .= '<a href="' . esc_url( $google_map_url ) . '"><img alt="" name="" src="' . esc_url( $img_src ) . '"></a>';

			$name_value['geolocation'] = $value;
		}

		return $name_value;
	}
}

new User_Registration_Geolocation_Process;