<?php
/**
 * User Registration Invite Codes Table List
 *
 * @since   1.0.0
 */

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

/**
 * Invite Codes list class.
 */
class UR_Invite_Codes_Table_List extends WP_List_Table {

	/**
	 * Form ID.
	 *
	 * @var int
	 */
	public $form_id;

	/**
	 * Initialize the codes table list.
	 */
	public function __construct() {

		$page   = ( isset( $_GET['page'] ) ) ? esc_attr( $_GET['page'] ) : false;
		$forms  = ur_get_all_user_registration_form();
		$latest = key( $forms );

		// @TODO::Verify Nonce
		$this->form_id = ! empty( $_REQUEST['form_id'] ) ? absint( $_REQUEST['form_id'] ) : $latest;

		parent::__construct(
			array(
				'singular' => 'invite-code',
				'plural'   => 'invite-codes',
				'ajax'     => false,
			)
		);
	}

	/**
	 * No items found text.
	 */
	public function no_items() {

		$add_code_link = esc_url( admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&section=add-new' ) );

		if ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'active' ) {
			esc_html_e( 'Whoops, it appears you do not have any active invite codes yet. Users may not be able to register to your site.', 'user-registration-invite-codes' );
			echo '<a href="' . $add_code_link . '"> Add now? </a>';
		} elseif ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'expired' ) {
			esc_html_e( 'It appears you donot have any expired codes.', 'user-registration-invite-codes' );
		} else {
			esc_html_e( 'Whoops, it appears you do not have invite codes yet. Users may not be able to register to your site.', 'user-registration-invite-codes' );
			echo '<a href="' . $add_code_link . '"> Add now? </a>';
		}
	}

	/**
	 * Get Active invite codes list table columns.
	 *
	 * @return array Columns.
	 */
	public function get_columns() {

		$columns           = array();
		$columns['cb']     = '<input type="checkbox" />';
		$columns['code']   = __( 'Invite Code', 'user-registration-invite-codes' );
		$columns['count']  = __( 'User Count', 'user-registration-invite-codes' );
		$columns['role']   = __( 'User Role', 'user-registration-invite-codes' );
		$columns['cdate']  = __( 'Created Date', 'user-registration-invite-codes' );
		$columns['edate']  = __( 'Expiry Date', 'user-registration-invite-codes' );
		$columns['status'] = __( 'Code Status', 'user-registration-invite-codes' );
		$columns['action'] = __( 'Action', 'user-registration-invite-codes' );

		return apply_filters( 'user_registration_invite_codes_list_table_columns', $columns );
	}

	/**
	 * Column cb.
	 *
	 * @param  array $item
	 *
	 * @return string
	 */
	public function column_cb( $items ) {
		return sprintf( '<input type="checkbox" name="%1$s[]" value="%2$s" />', $this->_args['singular'], $items['code'] );
	}

	/**
	 * Return invite code column.
	 *
	 * @param  object $items
	 *
	 * @return string
	 */
	public function column_code( $items ) {
		return isset( $items['code'] ) ? $items['code'] : '';
	}

	/**
	 * Return invite code count column.
	 *
	 * @param  object $items
	 *
	 * @return string
	 */
	public function column_count( $items ) {
		$count = $items['user_count'];
		$limit = $items['user_limit'];

		if ( ! empty( $count ) && ! empty( $limit ) ) {
			return $count . '/' . $limit;
		} elseif ( empty( $count ) && ! empty( $limit ) ) {
			return '0/' . $limit;
		} elseif ( ! empty( $count ) && empty( $limit ) ) {
			return $count . '/∞';
		} elseif ( empty( $count ) && empty( $limit ) ) {
			return '0/∞';
		}
	}

	/**
	 * Return role column.
	 *
	 * @param  object $items
	 *
	 * @return string
	 */
	public function column_role( $items ) {
		$all_roles = ur_get_default_admin_roles();
		$role      = $items['role'];

		return isset( $all_roles[ $role ] ) ? $all_roles[ $role ] : '-';
	}

	/**
	 * Return created date column.
	 *
	 * @param  object $posts
	 *
	 * @return string
	 */
	public function column_cdate( $items ) {
		return isset( $items['created_date'] ) ? $items['created_date'] : '-';
	}

	/**
	 * Return expiry date column.
	 *
	 * @param  object $posts
	 *
	 * @return string
	 */
	public function column_edate( $items ) {
		return ! empty( $items['expiry_date'] ) ? $items['expiry_date'] : '∞';
	}

	/**
	 * Return status column.
	 *
	 * @param  object $items
	 *
	 * @return string
	 */
	public function column_status( $items ) {
		$current_date = date( 'Y-m-d' );
		$expiry_date  = $items['expiry_date'];

		if ( ! isset( $expiry_date ) || empty( $expiry_date ) ) {
			return '<div class="ur-active">' . __( 'Active', 'user-registration-invite-codes' ) . '</div>';
		} elseif ( $current_date < $expiry_date ) {
			return '<div class="ur-active">' . __( 'Active', 'user-registration-invite-codes' ) . '</div>';
		} else {
			return '<div class="ur-expired">' . __( 'Expired', 'user-registration-invite-codes' ) . '</div>';
		}
	}

	/**
	 * Return action column.
	 *
	 * @param  object $items
	 *
	 * @return string
	 */
	public function column_action( $items ) {

		$status  = isset( $_REQUEST['status'] ) ? $_REQUEST['status'] : 'all';
		$action  = '<a href=" ' . esc_url( wp_nonce_url( admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=' . $status . '&action=edit_code&edit_code=' . $items['code'] . '&form_id=' . $this->form_id . '' ), 'edit_code' ) ) . '">' . __( 'Edit', 'user-registration-invite-codes' ) . '</a>';
		$action .= ' | ';
		$action .= '<a href=" ' . esc_url( wp_nonce_url( admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=' . $status . '&action=delete_code&delete_code=' . $items['code'] . '&form_id=' . $this->form_id . '' ), 'delete_code' ) ) . '">' . __( 'Delete', 'user-registration-invite-codes' ) . '</a>';

		return $action;
	}

	/**
	 * Get a list of sortable columns.
	 *
	 * @return array
	 */
	protected function get_sortable_columns() {
		return apply_filters(
			'user_registration_invite_codes_sortable_columns',
			array(
				'code'   => array( 'code', false ),
				'count'  => array( 'count', false ),
				'role'   => array( 'role', false ),
				'cdate'  => array( 'cdate', false ),
				'edate'  => array( 'edate', false ),
				'status' => array( 'status', false ),
			)
		);
	}

	/**
	 * Get bulk actions.
	 *
	 * @return array
	 */
	protected function get_bulk_actions() {

		return array(
			'delete' => __( 'Delete', 'user-registration-invite-codes' ),
		);
	}

	/**
	 * Get a list of hidden columns.
	 *
	 * @return array
	 */
	protected function get_hidden_columns() {
		return get_hidden_columns( $this->screen );
	}

	/**
	 * Set _column_headers property for table list
	 */
	protected function prepare_column_headers() {
		$this->_column_headers = array(
			$this->get_columns(),
			array(),
			$this->get_sortable_columns(),
		);
	}

	/**
	 * Prepare table list items.
	 *
	 * @global wpdb $wpdb
	 */
	public function prepare_items( $args = array() ) {

		$per_page     = $this->get_items_per_page( 'invite_codes_per_page', 20 );
		$columns      = $this->get_columns();
		$hidden       = $this->get_hidden_columns();
		$sortable     = $this->get_sortable_columns();
		$current_page = $this->get_pagenum();
		$total_items  = 0;

		$this->_column_headers = array( $columns, $hidden, $sortable );
		$this->items           = array();

		$options       = get_option( 'ur_invite_codes', array() );
		$current_date  = date( 'Y-m-d' );
		$active_items  = array();
		$expired_items = array();

		foreach ( $options as $code => $item ) {
			if ( $item['form'] != $this->form_id ) {
				continue;
			}

			$item['code']  = $code;
			$this->items[] = $item;
		}
		$total_items = count( $this->items );

		// All items.
		$this->items = array_slice( $this->items, ( ( $current_page - 1 ) * $per_page ), $per_page );

		foreach ( $this->items as $item ) {
			if ( $item['form'] != $this->form_id ) {
				continue;
			}

			if ( ! isset( $item['expiry_date'] ) || empty( $item['expiry_date'] ) ) {
				$active_items[] = $item;
			} elseif ( $current_date < $item['expiry_date'] ) {
				$active_items[] = $item;
			} else {
				$expired_items[] = $item;
			}
		}

		// Searched Items.
		if ( isset( $_REQUEST['page'] ) && $_REQUEST['page'] === 'ur-invite-codes' && ! empty( $_REQUEST['s'] ) ) {
			$this->items = ur_invite_codes_search( $this->items );
		}

		// Active items.
		if ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'active' ) {
			$this->items = $active_items;
		}

		// Expired Items.
		if ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'expired' ) {
			$this->items = $expired_items;
		}

		$this->set_pagination_args(
			array(
				'total_items' => $total_items,
				'per_page'    => $per_page,
				'total_pages' => ceil( $total_items / $per_page ),
			)
		);
	}

	/**
	 * Views of the list by status.
	 *
	 * @return string
	 */
	protected function get_views() {

		global $wpdb;
		$options       = get_option( 'ur_invite_codes', array() );
		$current_date  = date( 'Y-m-d' );
		$total_items   = 0;
		$active_items  = 0;
		$expired_items = 0;
		$total_class   = '';
		$active_class  = '';
		$expired_class = '';

		foreach ( $options as $item ) {
			if ( $item['form'] != $this->form_id ) {
				continue;
			}

			if ( ! isset( $item['expiry_date'] ) || empty( $item['expiry_date'] ) ) {
				$active_items++;
			} elseif ( $current_date < $item['expiry_date'] ) {
				$active_items++;
			} else {
				$expired_items++;
			}

			$total_items++;
		}

		// Active Items.
		if ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'active' ) {
			$active_class = 'current';
		}
		// Expired Items.
		elseif ( isset( $_REQUEST['status'] ) && $_REQUEST['status'] === 'expired' ) {
			$expired_class = 'current';
		}
		// Total Items.
		else {
			$total_class = 'current';
		}

		/* translators: %s: count */
		$status_links['all'] = "<a href='admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=all' class=" . $total_class . '>' . sprintf( _nx( 'All <span class="count">(%s)</span>', 'All <span class="count">(%s)</span>', $total_items, 'codes', 'user-registration-invite-codes' ), number_format_i18n( $total_items ) ) . '</a>';

		/* translators: %s: count */
		$status_links['active'] = "<a href='admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=active' class=" . $active_class . '>' . sprintf( _nx( 'Active <span class="count">(%s)</span>', 'Active <span class="count">(%s)</span>', $active_items, 'codes', 'user-registration-invite-codes' ), number_format_i18n( $active_items ) ) . '</a>';

		$status_links['expired'] = "<a href='admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=expired' class=" . $expired_class . '>' . sprintf( _nx( 'Expired <span class="count">(%s)</span>', 'Expired <span class="count">(%s)</span>', $expired_items, 'codes', 'user-registration-invite-codes' ), number_format_i18n( $expired_items ) ) . '</a>';

		return $status_links;
	}
}
